"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificationRequest = void 0;
const utils_1 = require("ethers/utils");
const polly_js_1 = __importDefault(require("polly-js"));
const origin_backend_core_1 = require("@energyweb/origin-backend-core");
const PreciseProofEntity_1 = require("./PreciseProofEntity");
const contracts_1 = require("../contracts");
class CertificationRequest extends PreciseProofEntity_1.PreciseProofEntity {
    constructor(id, configuration, isPrivate = false) {
        super(id, configuration);
        this.isPrivate = isPrivate;
        this.initialized = false;
    }
    static async create(fromTime, toTime, energy, deviceId, configuration, files, isVolumePrivate = false, forAddress) {
        if (energy.gt(origin_backend_core_1.MAX_ENERGY_PER_CERTIFICATE)) {
            throw new Error(`Too much energy requested. Requested: ${energy}, Max: ${origin_backend_core_1.MAX_ENERGY_PER_CERTIFICATE}`);
        }
        const { certificateClient } = configuration.offChainDataSource;
        const request = new CertificationRequest(null, configuration, isVolumePrivate);
        const { issuer } = configuration.blockchainProperties;
        const issuerWithSigner = issuer.connect(configuration.blockchainProperties.activeUser);
        await certificateClient.validateGenerationPeriod({ fromTime, toTime, deviceId });
        const success = await certificateClient.queueCertificationRequestData({
            deviceId,
            fromTime,
            toTime,
            energy,
            files
        });
        if (!success) {
            throw new Error('Unable to create CertificationRequest');
        }
        const data = await issuer.encodeData(fromTime, toTime, deviceId);
        const tx = await (forAddress
            ? issuerWithSigner.requestCertificationFor(data, forAddress, isVolumePrivate)
            : issuerWithSigner.requestCertification(data, isVolumePrivate));
        const { events: [certificationRequested] } = await tx.wait();
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        request.id = certificationRequested.args._id.toNumber();
        if (configuration.logger) {
            configuration.logger.info(`CertificationRequest ${request.id} created`);
        }
        return request.sync();
    }
    async sync() {
        const offChainData = await polly_js_1.default()
            .waitAndRetry([2000, 4000, 8000, 16000])
            .executeForPromise(() => this.configuration.offChainDataSource.certificateClient.getCertificationRequest(this.id));
        Object.assign(this, offChainData);
        this.initialized = true;
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`CertificationRequest ${this.id} synced`);
        }
        return this;
    }
    async approve() {
        const issuerInterface = new utils_1.Interface(contracts_1.IssuerJSON.abi);
        const validityData = issuerInterface.functions.isRequestValid.encode([this.id.toString()]);
        let approveTx;
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        if (this.isPrivate) {
            const commitment = {
                [this.owner]: this.energy
            };
            const { rootHash } = this.generateAndAddProofs(commitment);
            approveTx = await issuerWithSigner.approveCertificationRequestPrivate(this.id, rootHash, validityData);
        }
        else {
            approveTx = await issuerWithSigner.approveCertificationRequest(this.id, this.energy, validityData);
        }
        const { events } = await approveTx.wait();
        const certificateId = Number(events.find((log) => log.event === 'CertificationRequestApproved')
            .topics[3]);
        return certificateId;
    }
    async revoke() {
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const revokeTx = await issuerWithSigner.revokeRequest(this.id);
        await revokeTx.wait();
    }
    async requestMigrateToPublic(value) {
        if (!this.isPrivate) {
            throw new Error('Certificate is already public.');
        }
        const { issuer } = this.configuration
            .blockchainProperties;
        const activeUserAddress = await this.configuration.blockchainProperties.activeUser.getAddress();
        const commitment = {
            [activeUserAddress]: value
        };
        const { rootHash } = this.generateAndAddProofs(commitment);
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const tx = await issuerWithSigner.requestMigrateToPublic(this.id, rootHash);
        await tx.wait();
        return tx;
    }
    static async getAll(configuration) {
        const all = await configuration.offChainDataSource.certificateClient.getAllCertificationRequests();
        const certificationRequestPromises = all.map((certReq) => new CertificationRequest(certReq.id, configuration).sync());
        return Promise.all(certificationRequestPromises);
    }
}
exports.CertificationRequest = CertificationRequest;
//# sourceMappingURL=CertificationRequest.js.map